% std_backproj()-This function is to backproject STUDY cluster-selected ICs
%                to each subject's channels that are common across all subjects
%                in the STUDY. Optionally, users can also save the .set files
%                that have only selected and backprojected ICs. Use this function
%                after loading STUDY.
% 
% LIMITATION  -  Within-subject conditions should be contained in a single .set file
%                (i.e. do not separate .set files for each condition!)
%                Session not supported. Two within-subject conditions not supported.
%
% Usage:
%       std_backproj(STUDY, ALLEEG, EEG, clustInclude, clustExclude, savePath);
% 
%       Outputs are available in STUDY as it directly 'assignin' STUDY in the 'base'.  
%       This function is called by std_pop_backproj()
%
% Inputs:
%       STUDY - EEGLAB STUDY. 
%       ALLEEG - EEGLAB ALLEEG.
%       EEG - EEGLAB EEG.
%       clustInclude - Index numbers of STUDY clusters for backprojection. [integer(s)]
%       clustExclude - Index numbers of STUDY clusters to exclude from pvaf calculation. [integer(s)]
%       savePath - Optional. If provided, save new .set files with selected ICs as '(old_finename)_backproj'. 
%       createStudyBackproj - Whether to create STUDY.backproj. [Default: 1]
%
% Outputs:
%       These are stored under STUDY.backproj
%       includeIcList - Columns are [index in ALLEEG, IC index in the dataset, cluster index, group index]
%       excludeIcList - Same as above.
%       subjectList - List of the datasets included in the 'includeIcList'.
%       latencyInMillisecond - Same as EEG.times
%       backprojErp - dimensions are [channels, latency, datasetIndex, group(if any)]
%       outermostEnv - dimensions are [outermostEnvelopes, latency, datasetIndex, group(if any)]
%       backprojEnv - dimensions are [backprojectedEnvelopes, latency, datasetIndex, group(if any)]
%       erpPvaf - dimensions are [latency, datasetIndex]
%       channelList - channel names that are common across all the datasets submitted
%       betweenSubjectCondition (if 'group' is present) - levels of the groups selected in the current design
%       groupList (if 'group' is present) - dataset group index list
%       withinSubjectCondition (if it is present) - levels of the within-subject conditions selected in the current design
%       lastExportData (if data are exported) - columns are [latency,ERP,std]/[latency,outermostEnvelope,backprojectedEnvelope,pvaf] To compute the mean pvaf, average pvaf (6th column).

% Author: Makoto Miyakoshi JSPS/SCCN,INC,UCSD
% History:
% 01/05/2015 ver 8.0 by Makoto. Major renewal upon the request by Jens Bernhardsson. Automatically selects all channels available. erpPvaf is stored with full latency frames.
% 03/04/2014 ver 7.1 by Makoto. [] in the input box for the cluster number removed.
% 02/27/2014 ver 7.0 by Makoto. pop_selectevent -> pop_epoch (thanks to Lizzy Blundon). Available channel list output added. pvaf topo & channel added.
% 02/05/2014 ver 6.4 by Makoto. Help button added. Limitation and caution added.
% 06/28/2013 ver 6.3 by Makoto. Added fprintf('Saved %s/%s_backproj\n', pwd, dataName)
% 06/18/2013 ver 6.2 by Makoto. Clear STUDY.backproj timing optimized.
% 01/28/2013 ver 6.1 by Makoto. Combined conditions supported.
% 01/17/2013 ver 6.0 by Makoto. Bug fixed (thanks to Anthony Rissling).
% 01/04/2013 ver 5.1 by Makoto. Help added.
% 01/04/2013 ver 5.0 by Makoto. Handles multi-channel multi-cluser requests. Backprojected dataset can be saved.
% 10/26/2012 ver 4.0 by Makoto. Renewed for a plugin.
% 10/24/2012 ver 3.2 by Makoto. clear tmp actualindex added.
% 10/23/2012 ver 3.1 by Makoto. icList included in the final output. icList1 now refers to STUDY.design.
% 10/12/2012 ver 3.0 by Makoto. Multiple cluster backprojection supported
% 07/12/2012 ver 2.0 by Makoto. Serious bug fixed (channel number was used for IC indICes!) 
% 07/10/2012 ver 1.0 by Makoto. Test version created.

% Copyright (C) 2012 Makoto Miyakoshi JSPS/SCCN,INC,UCSD
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General PublIC LICense as published by
% the Free Software Foundation; either version 2 of the LICense, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General PublIC LICense for more details.
%
% You should have received a copy of the GNU General PublIC LICense
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1.07  USA

function std_backproj(STUDY, ALLEEG, EEG, clustInclude, clustExclude, savePath, createStudyBackproj)

%%%%%%%%%%%%%%%%%%%
%%% input check %%%
%%%%%%%%%%%%%%%%%%%
if isempty(clustInclude)
    error('Enter the cluster numbers to backproject.')
end

if isempty(savePath)
    disp('New .set files with selected ICs backprojected will NOT be created.')
else
    disp(['New .set files with selected ICs backprojected will be created under ' savePath])
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% find common channels available across all subjects %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for n = 1:length(ALLEEG)-1
    if n == 1
        commonChanList = {ALLEEG(1,n).chanlocs.labels};
    end
    currentChanList = {ALLEEG(1,n+1).chanlocs.labels};
    commonChanList  = intersect(commonChanList, currentChanList);
    if isempty(commonChanList)
        error('No common channel!')
    end
end
disp(sprintf('%d channels are commonly available across all datasets.',length(commonChanList)))
clear currentChanList

clustInclude = str2num(clustInclude);
if any(clustExclude)
    clustExclude = str2num(clustExclude);
else
    clustExclude = 0;
end
clustUse = nonzeros([clustInclude clustExclude])';
backprojChan  = commonChanList;
clear commonChanList

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% delete old STUDY.backproj variables %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isfield(STUDY, 'backproj')
    STUDY = rmfield(STUDY, 'backproj');
    disp('Old STUDY.backproj is deleted for new results.')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% create subj & IC list: no group %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
designLabel = {STUDY.design(STUDY.currentdesign).variable.label};
groupFieldIdx = find(strcmp(designLabel, 'group'));
if isempty(groupFieldIdx)
    % pre-allocate memory
    perGroupLength = 0;
    for clust = 1:length(clustUse)
        tmpLength = length(STUDY.cluster(1,clustUse(clust)).setinds{1,1});
        perGroupLength = perGroupLength + tmpLength;
    end
    allSetIc = zeros(perGroupLength, 3);
    
    % extract indices for setfiles and ICs
    listCounter = 0;
    for clust = 1:length(clustUse)
        tmpSetInd = STUDY.cluster(1,clustUse(clust)).setinds{1,1};
        for m = 1:length(tmpSetInd)
            listCounter = listCounter+1;
            allSetIc(listCounter,1) = STUDY.design(STUDY.currentdesign).cell(tmpSetInd(1,m)).dataset; %#ok<*AGROW> % dataset number
            allSetIc(listCounter,2) = STUDY.cluster(1,clustUse(clust)).allinds{1,1}(1,m); % IC number
            allSetIc(listCounter,3) = clustUse(clust); % cluster number
        end
    end
    
    % sort the results into the order of the setfile index
    tmpSetInd    = allSetIc(:,1);
    [~,sortIndex] = sort(tmpSetInd);
    sortedList = [allSetIc(sortIndex,:) repmat(1, [length(tmpSetInd) 1])];
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% create subj & IC list: with group %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
else
    % detect 'group' in the STUDY.design.variable
    STUDY.backproj.betweenSubjectCondition = STUDY.design(STUDY.currentdesign).variable(1,groupFieldIdx).value;
    
    % pre-allocate memory: one cannot use length(STUDY.group) because they may be combined by user
    if groupFieldIdx == 1
        combinedGroupValue = STUDY.design(STUDY.currentdesign).variable(1,1).value;
    else
        combinedGroupValue = STUDY.design(STUDY.currentdesign).variable(1,2).value;
    end
    perGroupLength = zeros(1, length(combinedGroupValue));
    
    for group = 1:length(combinedGroupValue)
        for clust = 1:length(clustUse)
            switch groupFieldIdx
                case 1
                    tmpLength = length(STUDY.cluster(1,clustUse(clust)).setinds{group,1});
                case 2
                    tmpLength = length(STUDY.cluster(1,clustUse(clust)).setinds{1,group});
            end
            perGroupLength(1,group) = perGroupLength(1,group) + tmpLength;
        end
    end
    for group = 1:length(combinedGroupValue)
        allSetIc{1,group} = zeros(perGroupLength(1,group), 3);
    end
    
    % extract indices for setfiles and ICs
    for group = 1:length(combinedGroupValue)
        listCounter = 0;
        for clust = 1:length(clustUse)
            switch groupFieldIdx
                case 1
                    tmpSetInd = STUDY.cluster(1,clustUse(clust)).setinds{group,1};
                case 2
                    tmpSetInd = STUDY.cluster(1,clustUse(clust)).setinds{1,group};
            end
            for m = 1:length(tmpSetInd)
                listCounter = listCounter+1;
                allSetIc{1,group}(listCounter,1) = STUDY.design(STUDY.currentdesign).cell(tmpSetInd(m)).dataset; %#ok<*AGROW> % dataset number
                switch groupFieldIdx
                    case 1
                        allSetIc{1,group}(listCounter,2) = STUDY.cluster(1,clustUse(clust)).allinds{group,1}(1,m); % IC number
                    case 2
                        allSetIc{1,group}(listCounter,2) = STUDY.cluster(1,clustUse(clust)).allinds{1,group}(1,m); % IC number
                end
                allSetIc{1,group}(listCounter,3) = clustUse(clust); % cluster number
            end
        end
    end
    
    % sort the results into the order of the setfile index
    for group = 1:length(combinedGroupValue)
        tmpSetInd    = allSetIc{1,group}(:,1);
        [~,sortIndex] = sort(tmpSetInd);
        allSetIc{1,group} = allSetIc{1,group}(sortIndex,:); 
        tmpList = [allSetIc{1,group} repmat(group, [length(allSetIc{1,group}) 1])];
        % sortedList == [subjID IC cluster group];
        if group == 1
            sortedList = tmpList;
        else
            sortedList = [sortedList; tmpList];
        end
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% calculate unique subjects %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
includeIcIdx = find(ismember(sortedList(:,3),clustInclude));
includeList  = sortedList(includeIcIdx,:);
STUDY.backproj.includeIcList = includeList;
if any(clustExclude)
    excludeIcIdx = find(~ismember(sortedList(:,3),clustInclude));
    excludeList  = sortedList(excludeIcIdx,:);
    STUDY.backproj.excludeIcList = excludeList;
end
[~,uniqueSubjIdx]  = unique(includeList(:,1));
uniqueSubj = includeList(uniqueSubjIdx,:);
uniqueSubj = uniqueSubj(:,[1 3]);
STUDY.backproj.subjectList   = {ALLEEG(uniqueSubj(:,1)).setname}';
STUDY.backproj.latencyInMillisecond = ALLEEG(1,1).times;
if any(groupFieldIdx)
    STUDY.backproj.groupList = includeList(uniqueSubjIdx,4);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% process each subject %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
waitBarHandle = waitbar(0,'Cluster ICs are being backprojected...');
waitBarCount  = 0;

% check STUDY variables
withinSubjectConditionPresent = [any(designLabel{1,1})& ~strcmp(designLabel{1,1},'group') any(designLabel{1,2})& ~strcmp(designLabel{1,2},'group')];

% store backprojection channel
STUDY.backproj.channelList   = backprojChan;

% pre-allocate memory-if there is within-subject condition
if createStudyBackproj == 1
    if any(sum(withinSubjectConditionPresent))
        tmpValue = STUDY.design(STUDY.currentdesign).variable(find(withinSubjectConditionPresent)).value;
        STUDY.backproj.withinSubjectCondition = tmpValue;
        withinSubjectConditionLabel = STUDY.design(STUDY.currentdesign).variable(find(withinSubjectConditionPresent)).label;
        withinSubjectConditionNum   = length(tmpValue);
        STUDY.backproj.outermostEnv = single(zeros(2, length(ALLEEG(1,1).times), length(uniqueSubj), withinSubjectConditionNum));
        STUDY.backproj.backprojEnv  = single(zeros(2, length(ALLEEG(1,1).times), length(uniqueSubj), withinSubjectConditionNum));
        STUDY.backproj.erpPvaf      = single(zeros(length(ALLEEG(1,1).times), length(uniqueSubj), withinSubjectConditionNum));
        
        % pre-allocate memory-if there is NO within-subject condition
    else
        STUDY.backproj.outermostEnv = single(zeros(2, length(ALLEEG(1,1).times), length(uniqueSubj)));
        STUDY.backproj.backprojEnv  = single(zeros(2, length(ALLEEG(1,1).times), length(uniqueSubj)));
        STUDY.backproj.erpPvaf      = single(zeros(length(ALLEEG(1,1).times), length(uniqueSubj)));
    end
end

% process each subject
for n = 1:length(uniqueSubj)
    % process waitbar
    waitBarCount = waitBarCount+1;
    waitbar(waitBarCount/size(uniqueSubj,1), waitBarHandle)
    % load subject data & convert double to single
    [~, EEG] = pop_newset(ALLEEG, EEG, 1, 'retrieve', uniqueSubj(n,1), 'study', 1);
    EEG.data   = single(EEG.data);
    EEG.icaact = [];
    % check data dimension
    if size(EEG.data)<=2
        error('EEG data must be epoched.')
    end
    % keep only selected ICs
    subjIdx = find(includeList(:,1)==uniqueSubj(n,1));
    icToKeep  = includeList(subjIdx,2);
    EEG_backproj = pop_subcomp(EEG, setdiff([1:size(EEG.icaweights,1)], icToKeep), 0);
    % save
    if any(savePath)
        pop_saveset(EEG_backproj, 'filename', [EEG.filename(1:end-4) '_backproj'], 'filepath', savePath);
        if createStudyBackproj == 0
            continue
        end
    end
    
    % backprojChan
    EEG_backproj_commonCh = pop_select( EEG_backproj,'channel',backprojChan);
    EEG_backproj_commonCh.icaact = [];
    clear EEG_backproj
    
    EEG_commonCh = pop_select( EEG, 'channel', backprojChan);
    EEG_commonCh.icaact = [];

    switch sum(withinSubjectConditionPresent)
        % No within-subject condition
        case 0
            % store backprojected ERP for all the channels common across all subjects
            STUDY.backproj.backprojErp(:,:,n) = single(mean(EEG_backproj_commonCh.data,3));
            % prepare cluster-excluded 'clean' EEG
            if any(clustExclude) & any(find(excludeList(:,1)==uniqueSubj(n,1)))
                subjIdx = find(excludeList(:,1)==uniqueSubj(n,1));
                icToExclude = excludeList(subjIdx,2);
                EEG_commonCh_clustExcluded = pop_subcomp(EEG_commonCh, icToExclude, 0);
            else
                EEG_commonCh_clustExcluded = EEG_commonCh;
            end
            % compute pvaf
            STUDY.backproj.outermostEnv(:,:,n) = single([max(mean(EEG_commonCh_clustExcluded.data,3),[],1); min(mean(EEG_commonCh_clustExcluded.data,3),[],1)]);
            STUDY.backproj.backprojEnv(:,:,n)  = single([max(mean(EEG_backproj_commonCh.data,3),[],1); min(mean(EEG_backproj_commonCh.data,3),[],1)]);
            STUDY.backproj.erpPvaf(:,n)        = single(100-100*var(mean(EEG_commonCh_clustExcluded.data,3)-mean(EEG_backproj_commonCh.data,3),0,1)./var(mean(EEG_commonCh_clustExcluded.data,3),0,1));

        % One within-subject condition
        case 1
            for numCondition = 1:length(tmpValue);
                tmpCondition = tmpValue(1,numCondition);
                % if multiple conditions are combined by STUDY
                if iscell(tmpCondition{1,1})
                    EEG_backproj_commonCh_condition = pop_selectevent( EEG_backproj_commonCh, withinSubjectConditionLabel, tmpCondition{1,1},'deleteevents','off','deleteepochs','on','invertepochs','off');
                    EEG_backproj_commonCh_condition.icaact = [];
                    EEG_commonCh_condition = pop_selectevent( EEG_commonCh, withinSubjectConditionLabel, tmpCondition{1,1},'deleteevents','off','deleteepochs','on','invertepochs','off');
                    EEG_commonCh_condition.icaact = [];
                % if single condition
                else
                    EEG_backproj_commonCh_condition = pop_selectevent( EEG_backproj_commonCh, withinSubjectConditionLabel, tmpCondition,'deleteevents','off','deleteepochs','on','invertepochs','off');
                    EEG_backproj_commonCh_condition.icaact = [];
                    EEG_commonCh_condition = pop_selectevent( EEG_commonCh, withinSubjectConditionLabel, tmpCondition,'deleteevents','off','deleteepochs','on','invertepochs','off');
                    EEG_commonCh_condition.icaact = [];
                end
                % store backprojected ERP for all the channels common across all subjects
                STUDY.backproj.backprojErp(:,:,n,numCondition) = single(mean(EEG_backproj_commonCh_condition.data, 3));
                
                % prepare cluster-excluded 'clean' EEG
                if any(clustExclude) & any(find(excludeList(:,1)==uniqueSubj(n,1)))
                    subjIdx = find(excludeList(:,1)==uniqueSubj(n,1));
                    icToExclude = excludeList(subjIdx,2);
                    EEG_commonCh_clustExcluded = pop_subcomp(EEG_commonCh_condition, icToExclude, 0);
                else
                    EEG_commonCh_clustExcluded = EEG_commonCh_condition;
                end
                % compute pvaf
                STUDY.backproj.outermostEnv(:,:,n,numCondition) = single([max(mean(EEG_commonCh_clustExcluded.data,3),[],1); min(mean(EEG_commonCh_clustExcluded.data,3),[],1)]);
                STUDY.backproj.backprojEnv(:,:,n,numCondition)  = single([max(mean(EEG_backproj_commonCh_condition.data,3),[],1); min(mean(EEG_backproj_commonCh_condition.data,3),[],1)]);
                STUDY.backproj.erpPvaf(:,n,numCondition)        = single(100-100*var(mean(EEG_commonCh_clustExcluded.data,3)-mean(EEG_backproj_commonCh_condition.data,3),0,1)./var(mean(EEG_commonCh_clustExcluded.data,3),0,1));
            end
        case 2
            error('Two within-subject conditions not supported. Please email Makoto Miyakoshi if you have a request.')
    end
end

close(waitBarHandle)
if createStudyBackproj == 0
    disp('Backprojection done.')
    return
else
    assignin('base', 'STUDY', STUDY)
    disp('Backprojection done. Data are stored at STUDY.backproj.')
end